# Generated by Django 2.1.2 on 2018-11-07 12:16

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='AccessToken',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('token', models.CharField(max_length=30)),
                ('created_at', models.DateField(auto_now_add=True)),
            ],
            options={
                'get_latest_by': 'created_at',
            },
        ),
    ]
